
// 3-digit LCD glass driver for PIC16F628
// Juha Niinikoski 04.01.2008
// leading zero blancking 28.09.2011


const unsigned char chgenN[16] =
{0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,0x7F,0x6F,0x77,0x7c,0x39,0x5E,0x79,0x71};
//  0    1    2    3    4    5    6    7    8    9    A    b    C    d    E    F     

#define DPP		0x80						// decimal point
#define MINUS	0x40						// - sign
#define BLANK	0x00						// blank


unsigned char out_buffer[3];				// 3 char for lcd


/* constant texts */

// constant text "---"
void prompt_lcd(void)
	{
	int i;
	for(i=0;i<3;i++)
		out_buffer[i] = MINUS;			// ---
	}


/* output int -99 to 999 to LCD */

void put_int(int i, char dp)
	{
	char sign;
	int mag;
	
	if(i > 999)										// check limits
		i = 999;
		
	if(i < -99)
		i = -99;

	sign = 0;
	if(i < 0)										// if negative, set sign & invert
		{
		i = 0 - i;
		sign = 1;
		}

	mag = i;										// save magnitude for leading zero blancking

// units
		out_buffer[0] = chgenN[(i % 10)];			// start from LSB
		i = i / 10;

// tens
		if(mag > 9)
			{
			out_buffer[1] = chgenN[(i % 10)];
			if((dp & 1) != 0)
				out_buffer[1] = out_buffer[1] | DPP;	// set DP
			}
		else
			out_buffer[1] = 0;						// blanc zero

		i = i / 10;	

// hundreds	
		if(mag > 99)
			{	
			if(sign == 0)								// positive
				{
				out_buffer[2] = chgenN[(i % 10)];
				if((dp & 2) != 0)
					out_buffer[2] = out_buffer[2] | DPP; // set DP		
				}
			else										// negative				
				out_buffer[2] = MINUS;
			}
		else
			out_buffer[2] = 0;						// blanc zero

	asm("clrwdt");
	}




/* LCD glass driver. Called every ??ms under interrupt */
/* out_buffer[0] bit D7 = glass common, must allways be 0 */ 

void xfer_lcd_buffer(void)	//
	{
	static char pol;				// polarity flip flop
	char x;
	int y;
	unsigned char tmp;

	if(pol == 0)
		pol = 1;					// flip
	else
		pol = 0;					// flop

	LCD_PORT = LCD_PORT & ~LCD_CLOCK;	// clock=0
	LCD_PORT = LCD_PORT & ~LCD_STR;		// strobe=0

	for(y=0; y<3; y++)				// do 3 rounds
		{
		tmp = out_buffer[y];		// get byte for this round

		for(x=0; x<8; x++)			// shift out 8 bits msb first
			{
			if(pol == 0)			// direct output
				{
				if(tmp & 0x80)		// evaluate bit
					LCD_PORT = LCD_PORT | LCD_BIT;	// 1
				else
					LCD_PORT = LCD_PORT & ~LCD_BIT;	// 0
				}
			else					// inverted output
				{
				if(tmp & 0x80)		// evaluate bit
					LCD_PORT = LCD_PORT & ~LCD_BIT;	// 0
				else
					LCD_PORT = LCD_PORT | LCD_BIT;	// 1
				}

			tmp  = tmp  << 1;		// shift for next round

			LCD_PORT = LCD_PORT | LCD_CLOCK;			// generate shift in clock
			asm("nop");
			LCD_PORT = LCD_PORT & ~LCD_CLOCK;
			} 		
		}
	asm("nop");
	LCD_PORT = LCD_PORT |LCD_STR;						// generate latch clock
	asm("nop");
	LCD_PORT = LCD_PORT & ~LCD_STR;

	}
